#include <system.h>
#include "AutoBevLib.h"

#pragma DATA  _CONFIG1H, 0b11001001
#pragma DATA  _CONFIG2H, _WDT_OFF_2H
#pragma DATA  _CONFIG3H, _MCLRE_ON_3H 
#pragma DATA  _CONFIG4L, _LVP_OFF_4L

//////////////////////////////////////////
//	AutoBev Senior Design Project		//
//										//
//	AutoBev.c							//
//  Main code for the project			//
//////////////////////////////////////////

//--// Voltatile bit Declarations


	volatile bit toint@INTCON.2; 		// Timer 0 Interrupt Flag
	volatile bit intedge1@INTCON2.5; 	// Interrupt on rising edge
	volatile bit int1IF@INTCON3.0; 		// Int 1 interrupt flag to Detect Flow Sensor Pulses	
	volatile bit cren@RCSTA.4;  		// For enabling reception of the USART interrupt
	volatile bit rcif@PIR1.5;			// Flag to detect received data

//--// Global Interrupt Semiphores

	bool increaseVolume = false; 		// Flow Sensor Pulse Interrupt
	
//--// Other Semiphores
	bool volFlag=false; 				// Flag For Reaching Volume
	bool cupPresent = false; 			// Cup Present for Dispensing


///////////////////////////////////////////
// 	Function: interrupt
//	Purpose:  The interrupt service routine
//	Input:	  Void
//  Output:   Void
///////////////////////////////////////////
void interrupt(void){		
	if(int1IF){
		increaseVolume=true; 			// Indicate that there is something to do 
		int1IF=0; 						// reset interrupt flag			
	}
}

//--// MAIN FUNCTION
void main(void)
{
	//--// Setup I/O Pins
		
		intcon=0b11000000;				// enable global interrupts
		intcon2=0b00110000;				// enable intedge interrupt
		intcon3=0b01001000; 			// enable interrupt flag				
		adcon1=0b00011111;				// make ports digital inputs		
		cmcon=0b00000111;				// configure the comparators
		osccon = 0b01110000;			// set up oscillator control		
	
		trisb.0 = 1; 
		trisb.1 = 1;					// Pulse input from flow sensor	 	
		trisb.2 = 0; 
		trisb.3 = 0; 					// Output to solenoid valve
		trisb.4 = 1; 					// Keg override button 
		trisb.5 = 1; 					// Keg replacement override
		trisc.0=0;	
		trisc.1=0;
		trisd.0 = 1; 
		trisd.1 = 1; 					// Input from limit switch	
		trisd.6 = 0; 
		trisd.7 = 0; 
		
		latb.2 = 0;	
		latb.3 = 0; 					// Close solenoid to begin		
		latc.0 = 0;	
		latc.1 = 0;	
		latd.6=0;						// Output for simulate pulses 
		latd.7=0; 						// Output for testing		

	
	//-// General Variables
		int inc = 1; 	   				// set increment value 
		int limit;  					// set volume limit (# of pulses before increment)	
		int count_2= 0;					// for reporting status back to PC
		char PIC_status; 				// byte for status of microcontroller
		char upper; 					// for upper 8 bits of volume poured
		char lower;						// for lower 8 bits of volume pourted
		long volume = 0;				// Volume Dispensed
		bool keepPouring = false;		// indicates it is ok to keep solenoid valve open
		unsigned char receivedByte;		// status byte from PC
		unsigned short volumeLimit;		// Volume Limit in Pulses			
	
	//--// Reset Status Byte
		PIC_status=0;	
		
	//--// USART Initialization
		init_usart();					// Initializes USART @ 9.6k baud rate

	
	//--// USART Initialization// 
	while(1){
		//--// Instruction Checking Routine
		char receivedByte = receiveByte();				// Received byte from PC
		
		if (!(((receivedByte>>0))&0x01)) {				// Bit 0 cleared indicates error				
				PIC_status.0=0;
				volume = 0;
				sendByte(PIC_status);					// Send error Status to PC
		}
		//--// Size Check Routine
		else if (!(((receivedByte>>1))&0x01)) { 		// Bit 1 cleared indicates new order
					if (!((receivedByte>>3))&0x01) {
						volumeLimit = 29810; 			// Pulses in 3 pitchers (180 oz)
						volume = 0;						// Initialize Volume				
						volFlag = false;						
						PIC_status.0=1;				
						PIC_status.1=1;
						PIC_status.2=1;
						sendByte(PIC_status);
					}
						
					else if (((receivedByte>>4))&0x01) {
						volumeLimit = 331;//1988; 			// Pulses in 2//12 oz						
						volume = 0;
						volFlag = false;
						PIC_status.0=1;
						PIC_status.1=1;
						PIC_status.2=1;
						sendByte(PIC_status);
					}
					else if (((receivedByte>>5))&0x01) {
						volumeLimit = 663;//2650; 			// Pulses in 4//16 oz
						volume= 0;
						volFlag = false;
						PIC_status.0=1;
						PIC_status.1=1;
						PIC_status.2=1;	
						sendByte(PIC_status);
					}
					else if (((receivedByte>>6))&0x01) {
						volumeLimit = 994;//9937; 			// Pulses in 6//60 oz
						volume = 0;
						volFlag = false;
						PIC_status.0=1;
						PIC_status.1=1;
						PIC_status.2=1;
						sendByte(PIC_status);
					}
					else { 								// No size selected--an error in the byte sent
						PIC_status.0=0;
						PIC_status.1=0;
						PIC_status.2=0;
						sendByte(PIC_status);
					}
				
			} 
			//--// End Size Setting Routine
			
			//--// Start/ Stop Routine
			else if (((receivedByte>>1))&0x01){			// Bit 1 set indicates start or stop pouring
									
					if (((receivedByte>>2))&0x01){		// Bit 2 set indicates starting to pour						
						PIC_status.0=1;
						PIC_status.1=1;		
										
						if ((portd.0==true)){			// Check for cup present							
							latb.2=1; 					// Open solenoid valve
							keepPouring = true;			// Allow for pouring
							PIC_status.2=1;				// Cup Present
							PIC_status.0=1;				// No error in pouring							
						}
						else {
							latb.2=0; 					// Keep valve closed
							PIC_status.2=0;
							PIC_status.0=0;				// no cup present, error in pouring										
						}
												
						volFlag = false;
						upper = volume >> 8;
						lower = volume & 0xff;					
						
					}
					else {								// Bit 2 cleared indicates stop pouring
						keepPouring = false;
						volFlag = false;
						latb.2 = 0;										
					}	
					sendByte(PIC_status);
					sendByte(upper);
					sendByte(lower);
						
			} 
			//--// End Start/ Stop Routine
			
			//--// Start/ Stop Routine
			if (((receivedByte>>7))&0x01){				// Bit 7 set indicates person has checked out
				keepPouring = false;
				volFlag = false;
				latb.2=0;
				volumeLimit = 0;
				volume = 0;
				PIC_status.0=1;
				PIC_status.1=0;
				PIC_status.2=0;	
				sendByte(PIC_status);		
			}			
			
			//--// While Loop to Pour until volume limit is reached
			while ((volFlag==false) && (keepPouring == true)) {		
			
				if (((portd.0))==true){// || (portb.4))){			// Check if cup is present
					keepPouring = true;
					latb.2=1; 							// open the solenoid valve
					PIC_status.0=1;
					PIC_status.1=1;
					PIC_status.2=1;					
				}				
				else {									// Cup is not present
					keepPouring = false;
					latb.2=0; 							// Close solenoid valve
					// update status here
					PIC_status.0=0;
					PIC_status.1=1;
					PIC_status.2=0;
					upper=volume >> 8;
					lower = volume & 0xff;					
					sendByte(PIC_status);
					sendByte(upper);
					sendByte(lower);				
				}				
				
				//--// Checking Async PC instruction
				if(rcif){								// Check for PC instruction			
					receivedByte = rcreg;					
					if (!(((receivedByte>>2))&0x01)){ 	// Bit 1 cleared means stop pouring				
						latb.2=0; 						// Close solenoid valve
						keepPouring=false;
						PIC_status.0=1;
						PIC_status.1=1;
						PIC_status.2=1;
					}					
					else { 								// Bit 1 set means start pouring
						keepPouring=true;
						latb.2=1;						// Keep valvue open
						PIC_status.0=1;
						PIC_status.1=1;
						PIC_status.2=0;
					}
				} 		
				
				//---// Interrupt handling routine			
				if (increaseVolume) {					
					volume = volume + inc; 				// Increase pulses received
					increaseVolume=false; 				// Reset interrupt flag
					//delay_ms(10); 						// delay used for testing					
					
					if (volume > volumeLimit) {			// If volume has been reached					
						latb.2=0; 						// Close solenoid valve
						volFlag=true; 					// Break out of while loop							
						upper = volume >> 8;
						lower = volume & 0xff;					
						PIC_status.0=1;					// Set status
						PIC_status.1=0;
						PIC_status.2=1;
						sendByte(PIC_status);			// Send status and pulses to PC
						sendByte(upper);
						sendByte(lower);
						volume = 0;						// Reinitialize variables
						volumeLimit = 0;
						keepPouring = false;
						PIC_status.0=1;
						PIC_status.1=1;
						PIC_status.2=1;
						
					} // End volume reached routine
						
				} // End interrupt handling routine				
					
				count_2=count_2+1;						// 
					
				//--// Micro to PC status Report Routine
				if (count_2>1000) {
					upper=volume >> 8;
					lower = volume & 0xff;					
					sendByte(PIC_status);				// Send status and pulses to PC
					sendByte(upper);
					sendByte(lower);
					count_2=0;
					latd.6= ~latd.6;					// Toggle d.6 to simulate pulses
				}				
					
			} // close volume pouring while loop
			
		}// End Main Loop
} //end main
		
	
