#include <system.h>

//////////////////////////////////////////
//	AutoBev								//
//										//
//	AutoBevLib.c						//
//  Serial functions for usart			//
//////////////////////////////////////////

// Function Prototypes are in AutoBev.h

// Global Volatile Bits

////////////////////////////////////////////////////////
//	Function: init_usart
//	Purpose:  initializes usart
//	Input:    void-set Baud Rate to 9600
//  Output:   Void
////////////////////////////////////////////////////////

void init_usart(void)
{
	spbrg = 51;				// Set correct spbrg for 9.6k rate
	/**/	//rcsta default is 00000000b
			//rcsta = 10000000b;
			//bit 7: serial port enable - 1
			//bit 6: 8 bit reception    - 0
			//bit 5: D/C                - x
			//bit 4: Asynchronous Mode  - 1
			//bit 3: D/C                - x
			//bit 2: No framing error.  - x
			//bit 1: No overrun error.  - x
			//bit 0: D/C                - x
   
   rcsta.4 = 1; 			//enables async reception
   rcsta.7 = 1; 			//Serial port enabled 
   
   
   	/**/	//trxsta default is 00000010b
			//txsta = 00100100b;		
			//bit 7: x					-x
			//bit 6: 8-bit transmit		-x
			//bit 5: transmit enable	-1
			//bit 4: asynch mode		-0
			//bit 3: D/C				-x
			//bit 2: high speed			-1
			//bit 1: trmt				-x
			//bit 0: Parity bit			-x
   
   txsta.4=0; 				//Asynchronous mode
   txsta.5=1; 				//Transmit enabled   
   txsta.2=1;				//High speed baud rate mode
}

////////////////////////////////////////////////////
//	Function: sendByte
//	Purpose:  puts a byte to terminal via usart
//  Input:    value -> character to be typed (ascii)
//  Output:   Void
////////////////////////////////////////////////////
/* send ascii character to terminal*/
void sendByte(char value)
{	
	volatile bit txen@TXSTA.5;		// define transmit enable bit
	volatile bit trmt@TXSTA.1;		// define tranmist SR status bit
	txen = 1;						// enable transmission
	while(true) 
	{ 
		if(trmt)
		{
			txreg = value;			// put data in transmission reg
			return;
		}
	}		
}
///////////////////////////////////////////////////////
//	Function: receiveByte
//	Purpose:  gets a characeter from terminal via usart
//	Input:	  Void
//  Output:   The character present at the terminal
///////////////////////////////////////////////////////


char receiveByte(void)
{
	volatile bit cren@RCSTA.4;		// define enable reception bit
	volatile bit rcif@PIR1.5;		// define received flag
	char value;						// allocate memory for byte of data
	
	cren = 1;						// enable reception
	
	while(true)
	{
		if(rcif)					// wait till recieve is clear
		{
			value = rcreg;			// get value
			//putc(value);  			// print gathered value to terminal
			return value;
		}
	}
}	
